@@ -1,4 +1,4 @@
-# $Id: LibXSLT.pm 186 2006-09-19 09:02:23Z pajas $
+# $Id: LibXSLT.pm 192 2006-11-17 21:01:06Z pajas $
 package XML::LibXSLT;
 
 use strict;
@@ -14,7 +14,7 @@ use Carp;
 
 require Exporter;
 
-$VERSION = "1.61";
+$VERSION = "1.62";
 
 require DynaLoader;
 
@@ -87,6 +87,16 @@ sub xpath_to_string {
 # callback functions                                                      #
 #-------------------------------------------------------------------------#
 
+sub security_callbacks {
+   my $self = shift;
+   my $scbclass = shift;
+
+   if ( defined $scbclass ) {
+      $self->{XML_LIBXSLT_SECPREFS} = $scbclass;
+   }
+   return $self->{XML_LIBXSLT_SECPREFS};
+}
+
 sub input_callbacks {
     my $self = shift;
     my $icbclass = shift;
@@ -232,6 +242,7 @@ sub parse_stylesheet {
                XML_LIBXSLT_OPEN_CB => $self->{XML_LIBXSLT_OPEN_CB},
                XML_LIBXSLT_READ_CB => $self->{XML_LIBXSLT_READ_CB},
                XML_LIBXSLT_CLOSE_CB => $self->{XML_LIBXSLT_CLOSE_CB},
+               XML_LIBXSLT_SECPREFS => $self->{XML_LIBXSLT_SECPREFS},
              };
 
     return bless $rv, "XML::LibXSLT::StylesheetWrapper";
@@ -259,6 +270,7 @@ sub parse_stylesheet_file {
                XML_LIBXSLT_OPEN_CB => $self->{XML_LIBXSLT_OPEN_CB},
                XML_LIBXSLT_READ_CB => $self->{XML_LIBXSLT_READ_CB},
                XML_LIBXSLT_CLOSE_CB => $self->{XML_LIBXSLT_CLOSE_CB},
+               XML_LIBXSLT_SECPREFS => $self->{XML_LIBXSLT_SECPREFS},
              };
 
     return bless $rv, "XML::LibXSLT::StylesheetWrapper";
@@ -280,6 +292,16 @@ use vars qw($MatchCB $ReadCB $OpenCB $CloseCB);
 use XML::LibXML;
 use Carp;
 
+sub security_callbacks {
+   my $self = shift;
+   my $scbclass = shift;
+
+   if ( defined $scbclass ) {
+      $self->{XML_LIBXSLT_SECPREFS} = $scbclass;
+   }
+   return $self->{XML_LIBXSLT_SECPREFS};
+}
+
 sub input_callbacks {
     my $self     = shift;
     my $icbclass = shift;
@@ -391,6 +413,11 @@ sub _init_callbacks {
     }
     $self->XML::LibXSLT::lib_init_callbacks();
     $icb->init_callbacks();
+
+    my $scb = $self->{XML_LIBXSLT_SECPREFS};
+    if ( $scb ) {
+       $scb->init_callbacks();
+    }
 }
 
 sub _cleanup_callbacks {
@@ -400,6 +427,11 @@ sub _cleanup_callbacks {
     if ( defined $mcb ) {
         $self->{XML_LIBXSLT_CALLBACK_STACK}->unregister_callbacks( [$mcb] );
     }
+
+    my $scb = $self->{XML_LIBXSLT_SECPREFS};
+    if ( $scb ) {
+       $scb->cleanup_callbacks();
+    }
 }
 
 sub transform {
@@ -407,7 +439,7 @@ sub transform {
     my $doc;
 
     $self->_init_callbacks();
-    eval { $doc = $self->{XML_LIBXSLT_STYLESHEET}->transform(@_); };
+    eval { $doc = $self->{XML_LIBXSLT_STYLESHEET}->transform($self,@_); };
     $self->_cleanup_callbacks();
 
     my $err = $@;
@@ -423,7 +455,7 @@ sub transform_file {
     my $doc;
 
     $self->_init_callbacks();
-    eval { $doc = $self->{XML_LIBXSLT_STYLESHEET}->transform_file(@_); };
+    eval { $doc = $self->{XML_LIBXSLT_STYLESHEET}->transform_file($self,@_); };
     $self->_cleanup_callbacks();
 
     my $err = $@;
@@ -434,13 +466,119 @@ sub transform_file {
     return $doc;
 }
 
-sub output_string { shift->{XML_LIBXSLT_STYLESHEET}->output_string(@_) }
+sub output_string { shift->{XML_LIBXSLT_STYLESHEET}->_output_string($_[0],0) }
+sub output_as_bytes { shift->{XML_LIBXSLT_STYLESHEET}->_output_string($_[0],1) }
+sub output_as_chars { shift->{XML_LIBXSLT_STYLESHEET}->_output_string($_[0],2) }
 sub output_fh { shift->{XML_LIBXSLT_STYLESHEET}->output_fh(@_) }
 sub output_file { shift->{XML_LIBXSLT_STYLESHEET}->output_file(@_) }
 sub media_type { shift->{XML_LIBXSLT_STYLESHEET}->media_type(@_) }
 sub output_encoding { shift->{XML_LIBXSLT_STYLESHEET}->output_encoding(@_) }
 
 1;
+
+# XML::LibXSLT::Security Interface                                        #
+#-------------------------------------------------------------------------#
+package XML::LibXSLT::Security;
+
+use strict;
+use Carp;
+
+use vars qw(%OPTION_MAP %_GLOBAL_CALLBACKS);
+
+# Maps the option names used in the perl interface to the numeric values
+# used by libxslt.
+my %OPTION_MAP = (
+   read_file  => 1,
+   write_file => 2,
+   create_dir => 3,
+   read_net   => 4,
+   write_net  => 5,
+);
+
+%_GLOBAL_CALLBACKS = ();
+
+
+#-------------------------------------------------------------------------#
+# global callback                                                         #
+#-------------------------------------------------------------------------#
+sub _security_check {
+    my $option = shift;
+    my $retval = 1;
+
+    if ($option == 3) {
+       $retval = 0;             # Default create_dir to no access
+    }
+
+    if (exists $_GLOBAL_CALLBACKS{$option}) {
+       $retval = $_GLOBAL_CALLBACKS{$option}->(@_);
+    }
+
+    return $retval;
+}
+
+#-------------------------------------------------------------------------#
+# member functions and methods                                            #
+#-------------------------------------------------------------------------#
+
+sub new {
+    my $class = shift;
+    return bless {'_CALLBACKS' => {}}, $class;
+}
+
+# Add a callback for the given security option (read_file, write_file,
+# create_dir, read_net, write_net).
+#
+# To register a callback that handle network read requests:
+#   $scb->register_callback( read_net => \&callback );
+sub register_callback {
+   my $self = shift;
+   my $option = shift;
+   my $callback = shift;
+
+   unless ( exists $OPTION_MAP{$option} ) {
+      croak "Invalid security option '$option'. Must be one of: " .
+            join(', ', keys %OPTION_MAP) . ".";
+   }
+
+   if ( ref $callback eq 'CODE' ) {
+      $self->{_CALLBACKS}{ $OPTION_MAP{$option} } = $callback;
+   }
+   else {
+      croak "Invalid argument. The callback must be a reference to a subroutine";
+   }
+}
+
+# Removes the callback for the given security option. Causes the given option
+# to use the default security handler (which always allows the action).
+sub unregister_callback {
+   my $self = shift;
+   my $option = shift;
+
+   unless ( exists $OPTION_MAP{$option} ) {
+      croak "Invalid security option '$option'. Must be one of: " .
+            join(', ', keys %OPTION_MAP) . ".";
+   }
+
+   delete $self->{_CALLBACKS}{ $OPTION_MAP{$option} };
+}
+
+
+# make it so libxslt can use the callbacks
+sub init_callbacks {
+    my $self = shift;
+
+    %_GLOBAL_CALLBACKS = %{ $self->{_CALLBACKS} };
+}
+
+# reset libxslt callbacks
+sub cleanup_callbacks {
+    my $self = shift;
+
+    %_GLOBAL_CALLBACKS = ();
+}
+
+1;
+
 __END__
 
 =head1 NAME
@@ -480,7 +618,9 @@ methods. However either way you call them, it still sets global options.
 Each of the option methods returns its previous value, and can be called
 without a parameter to retrieve the current value.
 
-=head2 max_depth
+=over
+
+=item max_depth
 
   XML::LibXSLT->max_depth(1000);
 
@@ -490,14 +630,14 @@ recursion and detecting it. If your stylesheet or XML file requires
 seriously deep recursion, this is the way to set it. Default value is
 250.
 
-=head2 debug_callback
+=item debug_callback
 
   XML::LibXSLT->debug_callback($subref);
 
 Sets a callback to be used for debug messages. If you don't set this,
 debug messages will be ignored.
 
-=head2 register_function
+=item register_function
 
   XML::LibXSLT->register_function($uri, $name, $subref);
 
@@ -528,11 +668,15 @@ be a nodelist or a plain value - the code will just do the right thing.
 But only a single return value is supported (a list is not converted to
 a nodelist).
 
+=back
+
 =head1 API
 
 The following methods are available on the new XML::LibXSLT object:
 
-=head2 parse_stylesheet($doc)
+=over
+
+=item parse_stylesheet($doc)
 
 C<$doc> here is an XML::LibXML::Document object (see L<XML::LibXML>)
 representing an XSLT file. This method will return a 
@@ -540,16 +684,25 @@ XML::LibXSLT::Stylesheet object, or undef on failure. If the XSLT is
 invalid, an exception will be thrown, so wrap the call to 
 parse_stylesheet in an eval{} block to trap this.
 
-=head2 parse_stylesheet_file($filename)
+=item parse_stylesheet_file($filename)
 
 Exactly the same as the above, but parses the given filename directly.
 
-=head2 Input Callbacks
+=back
+
+=head1 Input Callbacks
 
 To define XML::LibXSLT or XML::LibXSLT::Stylesheet specific input
 callbacks, reuse the XML::LibXML input callback API as described in
 L<XML::LibXML::InputCallback(3)>.
 
+=head1 Security Callbacks
+
+To create security preferences for the transformation see
+L<XML::LibXSLT::Security>. Once the security preferences have been defined you
+can apply them to an XML::LibXSLT or XML::LibXSLT::Stylesheet instance using
+the C<security_callbacks()> method.
+
 =head1 XML::LibXSLT::Stylesheet
 
 The main API is on the stylesheet, though it is fairly minimal.
@@ -559,48 +712,62 @@ stylesheet object which you call the transform() method passing in a
 document to transform. This allows you to have multiple transformations
 happen with one stylesheet without requiring a reparse.
 
-=head2 transform(doc, %params)
+=over
+
+=item transform(doc, %params)
 
   my $results = $stylesheet->transform($doc, foo => "value);
 
 Transforms the passed in XML::LibXML::Document object, and returns a
 new XML::LibXML::Document. Extra hash entries are used as parameters.
 
-=head2 transform_file(filename, %params)
+=item transform_file(filename, %params)
 
   my $results = $stylesheet->transform_file($filename, bar => "value");
 
-=head2 output_string(result)
+=item output_string(result)
 
 Returns a scalar that is the XSLT rendering of the
 XML::LibXML::Document object using the desired output format
 (specified in the xsl:output tag in the stylesheet). Note that you can
 also call $result->toString, but that will *always* output the
-document in XML format, and in UTF8, which may not be what you asked
-for in the xsl:output tag.
+document in XML format which may not be what you asked for in the
+xsl:output tag.
 
-Note that the returned scalar is always bytes not characters, i.e. it
-is *not* marked with the UTF8 flag even if the desired output encoding
-was in fact UTF-8. If the output encoding was UTF-8 and you wish the
-scalar to be treated by Perl as characters, apply
-Encode::_utf8_on($result) on the returned scalar.
+Important note: The string returned by this function appears to Perl
+as characters if the output encoding was specified as UTF-8 and as
+bytes if no output encoding was specified or if the output encoding
+was different from UTF-8. See also C<output_as_bytes(result)> and
+C<output_as_chars(result)>.
 
-=head2 output_fh(result, fh)
+=item output_as_bytes(result)
+
+Like C<output_string(result)>, but always return the output as a byte
+string encoded in the output encoding specified in the stylesheet.
+
+=item output_as_chars(result)
+
+Like C<output_string(result)>, but always return the output as (UTF-8
+encoded) string of characters.
+
+=item output_fh(result, fh)
 
 Outputs the result to the filehandle given in C<$fh>.
 
-=head2 output_file(result, filename)
+=item output_file(result, filename)
 
 Outputs the result to the file named in C<$filename>.
 
-=head2 output_encoding
+=item output_encoding()
 
 Returns the output encoding of the results. Defaults to "UTF-8".
 
-=head2 media_type
+=item media_type()
 
 Returns the output media_type of the results. Defaults to "text/html".
 
+=back
+
 =head1 Parameters
 
 LibXSLT expects parameters in XPath format. That is, if you wish to pass
@@ -620,6 +787,112 @@ Obviously this isn't much fun, so you can make it easy on yourself:
 The utility function does the right thing with respect to strings in XPath,
 including when you have quotes already embedded within your string.
 
+
+=head1 XML::LibXSLT::Security
+
+Provides an interface to the libxslt security framework by allowing callbacks
+to be defined that can restrict access to various resources (files or URLs)
+during a transformation.
+
+The libxslt security framework allows callbacks to be defined for certain
+actions that a stylesheet may attempt during a transformation. It may be
+desirable to restrict some of these actions (for example, writing a new file
+using exsl:document). The actions that may be restricted are:
+
+=over
+
+=item read_file
+
+Called when the stylesheet attempts to open a local file (ie: when using the
+document() function).
+
+=item write_file
+
+Called when an attempt is made to write a local file (ie: when using the
+exsl:document element).
+
+=item create_dir
+
+Called when a directory needs to be created in order to write a file.
+
+NOTE: By default, create_dir is not allowed. To enable it a callback must be
+registered.
+
+=item read_net
+
+Called when the stylesheet attempts to read from the network.
+
+=item write_net
+
+Called when the stylesheet attempts to write to the network.
+
+=back
+
+=head2 Using XML::LibXSLT::Security
+
+The interface for this module is similar to XML::LibXML::InputCallback. After
+creating a new instance you may register callbacks for each of the security
+options listed above. Then you apply the security preferences to the
+XML::LibXSLT or XML::LibXSLT::Stylesheet object using C<security_callbacks()>.
+
+  my $security = XML::LibXSLT::Security->new();
+  $security->register_callback( read_file  => $read_cb );
+  $security->register_callback( write_file => $write_cb );
+  $security->register_callback( create_dir => $create_cb );
+  $security->register_callback( read_net   => $read_net_cb );
+  $security->register_callback( write_net  => $write_net_cb );
+
+  $xslt->security_callbacks( $security );
+   -OR-
+  $stylesheet->security_callbacks( $security );
+
+
+The registered callback functions are called when access to a resource is
+requested. If the access should be allowed the callback should return 1, if
+not it should return 0. The callback functions should accept the following
+arguments:
+
+=over
+
+=item $tctxt
+
+This is the transform context (XML::LibXSLT::TransformContext). You can use
+this to get the current XML::LibXSLT::Stylesheet object by calling
+C<stylesheet()>.
+
+  my $stylesheet = $tctxt->stylesheet();
+
+The stylesheet object can then be used to share contextual information between
+different calls to the security callbacks.
+
+=item $value
+
+This is the name of the resource (file or URI) that has been requested.
+
+=back
+
+If a particular option (except for C<create_dir>) doesn't have a registered
+callback, then the stylesheet will have full access for that action.
+
+=head2 Interface
+
+=over
+
+=item new()
+
+Creates a new XML::LibXSLT::Security object.
+
+=item register_callback( $option, $callback )
+
+Registers a callback function for the given security option (listed above).
+
+=item unregister_callback( $option )
+
+Removes the callback for the given option. This has the effect of allowing all
+access for the given option (except for C<create_dir>).
+
+=back
+
 =head1 BENCHMARK
 
 Included in the distribution is a simple benchmark script, which has two
@@ -644,10 +917,41 @@ would like to contribute them. Also if you get this running on Win32, I'd
 love to get a driver for MSXSLT via OLE, to see what we can do against
 those Redmond boys!
 
+=head1 LIBRARY VERSIONS
+
+For debugging purposes, XML::LibXSLT provides version information
+about the libxslt C library (but do not confuse it with the version
+number of XML::LibXSLT module itself, i.e. with
+C<$XML::LibXSLT::VERSION>). XML::LibXSLT issues a warning if the
+runtime version of the library is less then the compile-time version.
+
+=over 
+
+=item XML::LibXSLT::LIBXSLT_VERSION()
+
+Returns version number of libxslt library which was used to compile
+XML::LibXSLT as an integer. For example, for libxslt-1.1.15, it will
+return 10115.
+
+=item XML::LibXSLT::LIBXSLT_DOTTED_VERSION()
+
+Returns version number of libxslt library which was used to compile
+XML::LibXSLT as a string, e.g. "1.1.15".
+
+=item XML::LibXSLT::LIBXSLT_RUNTIME_VERSION()
+
+Returns version number of libxslt library to which XML::LibXSLT is
+linked at runtime (either dynamically or statically). For example, for
+example, for libxslt.so.1.1.15, it will return 10115.
+
+=back
+
 =head1 AUTHOR
 
 Matt Sergeant, matt@sergeant.org
 
+Security callbacks implementation contributed by Shane Corgatelli.
+
 Copyright 2001-2006, AxKit.com Ltd. All rights reserved.
 
 =head1 SEE ALSO
